package com.ai.abc.jpa.elastic.repository.impl;

import com.ai.abc.jpa.elastic.model.ElasticPage;
import com.ai.abc.jpa.elastic.model.PageInfo;
import com.ai.abc.jpa.elastic.model.TimeInfo;
import com.ai.abc.jpa.elastic.repository.ElasticEntityRepository;
import com.alibaba.fastjson.JSONObject;
import org.apache.commons.lang.StringUtils;
import org.apache.http.*;
import org.apache.http.message.BasicHeader;
import org.elasticsearch.action.index.IndexRequest;
import org.elasticsearch.action.index.IndexResponse;
import org.elasticsearch.action.search.SearchRequest;
import org.elasticsearch.action.search.SearchResponse;
import org.elasticsearch.action.search.SearchType;
import org.elasticsearch.client.RequestOptions;
import org.elasticsearch.client.RestClient;
import org.elasticsearch.client.RestClientBuilder;
import org.elasticsearch.client.RestHighLevelClient;
import org.elasticsearch.client.core.CountRequest;
import org.elasticsearch.client.core.CountResponse;
import org.elasticsearch.common.xcontent.XContentType;
import org.elasticsearch.index.query.BoolQueryBuilder;
import org.elasticsearch.index.query.QueryBuilder;
import org.elasticsearch.index.query.QueryBuilders;
import org.elasticsearch.index.query.TermsQueryBuilder;
import org.elasticsearch.rest.RestStatus;
import org.elasticsearch.search.SearchHit;
import org.elasticsearch.search.SearchHits;
import org.elasticsearch.search.aggregations.AggregationBuilders;
import org.elasticsearch.search.aggregations.Aggregations;
import org.elasticsearch.search.aggregations.BucketOrder;
import org.elasticsearch.search.aggregations.bucket.histogram.DateHistogramAggregationBuilder;
import org.elasticsearch.search.aggregations.bucket.histogram.DateHistogramInterval;
import org.elasticsearch.search.aggregations.bucket.histogram.Histogram;
import org.elasticsearch.search.aggregations.bucket.histogram.ParsedDateHistogram;
import org.elasticsearch.search.aggregations.bucket.terms.ParsedStringTerms;
import org.elasticsearch.search.aggregations.bucket.terms.Terms;
import org.elasticsearch.search.aggregations.bucket.terms.TermsAggregationBuilder;
import org.elasticsearch.search.aggregations.metrics.InternalValueCount;
import org.elasticsearch.search.aggregations.metrics.ParsedValueCount;
import org.elasticsearch.search.builder.SearchSourceBuilder;
import org.elasticsearch.search.sort.SortBuilder;
import org.elasticsearch.search.sort.SortBuilders;
import org.elasticsearch.search.sort.SortOrder;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.io.IOException;
import java.io.Serializable;
import java.util.*;

/**
 * 描述
 *
 * @author
 * @createDate
 */
@Component
public class ElasticEntityRepositoryImpl implements ElasticEntityRepository {
    public final Logger log = LoggerFactory.getLogger(ElasticEntityRepositoryImpl.class);

    private RestHighLevelClient client = null;

    @Override
    public void initServer(String esServer) {
        this.log.info("ElasticEntityRepositoryImpl esServer=" + esServer);
        this.getProducer(esServer,null,null);
    }

    @Override
    public void initServer(String esServer,String userName,String passWord) {
        this.log.info("ElasticEntityRepositoryImpl esServer=" + esServer + " userName:" + userName + " passWord:"+passWord);
        this.getProducer(esServer,userName,passWord);
    }


    private RestHighLevelClient getProducer(String elasticSearchServer,String userName,String passWord) {
        if (client == null) {
            String[] servers = elasticSearchServer.split(",");
            HttpHost[] httpHosts = new HttpHost[servers.length];
            for (int i = 0; i < servers.length; i++) {
                httpHosts[i] = HttpHost.create(servers[i]);
            }
            RestClientBuilder builder = RestClient.builder(httpHosts);

            Header[] headers = new Header[2];
            if(!StringUtils.isEmpty(userName) && !StringUtils.isEmpty(passWord)){
//                headers[0] = new BasicHeader(HttpHeaders.AUTHORIZATION, "Basic " + userName+":"+passWord);
                headers[0] = new BasicHeader(HttpHeaders.AUTHORIZATION, "Basic " + Base64.getUrlEncoder().encodeToString((userName + ":" + passWord).getBytes()));


                headers[1] = new BasicHeader("Content-Type", "application/json; charset=UTF-8");
                builder.setDefaultHeaders(headers);
            }

            client = new RestHighLevelClient(builder);
        }

        return client;
    }

    @Override
    public void purePersist(String index, String type, String messageBody) {
        IndexRequest request = new IndexRequest(index).source(messageBody, XContentType.JSON);
        try {
            IndexResponse indexResponse = client.index(request, RequestOptions.DEFAULT);
            log.debug(String.format("索引名称:%s; 更新内容:%s; 文档ID:%s; 状态：%s", indexResponse.getIndex(), messageBody,
                    indexResponse.getId(), indexResponse.status()));
        } catch (IOException e) {
            e.printStackTrace();
            //LOGGER.error(e.getMessage(), e);
        }
    }

    @Override
    public <T extends Serializable> ElasticPage<T> findEntity(String connectionName, Map<String, Object> hashMap, String startDate, String stopDate, PageInfo pageInfo) {
        SearchRequest searchRequest = new SearchRequest();
        SearchSourceBuilder sourceBuilder = new SearchSourceBuilder();

        // 处理查询条件
        List<TermsQueryBuilder> list = new ArrayList<>();
        if (hashMap !=null && !hashMap.isEmpty()) {
            for (Map.Entry<String, Object> entry : hashMap.entrySet()) {
                String mapKey = entry.getKey();
                if (entry.getValue() != null) {
                    String mapValue = entry.getValue().toString();
                    TermsQueryBuilder termsQueryBuilder = QueryBuilders.termsQuery(mapKey, mapValue);
                    list.add(termsQueryBuilder);
                }
            }
        }

        BoolQueryBuilder boolQueryBuilder = QueryBuilders.boolQuery();
        //处理开始结束时间
        if (StringUtils.isNotEmpty(startDate) && StringUtils.isNotEmpty(stopDate)) {
            boolQueryBuilder.must(QueryBuilders.rangeQuery("create_date").gte(startDate).lt(stopDate));
        } else if (StringUtils.isNotEmpty(startDate)) {
            boolQueryBuilder.must(QueryBuilders.rangeQuery("create_date").gte(startDate));
        } else if (StringUtils.isNotEmpty(stopDate)) {
            boolQueryBuilder.must(QueryBuilders.rangeQuery("create_date").lt(stopDate));
        }

        //处理其他条件
        for (TermsQueryBuilder termsAggregationBuilder : list) {
            boolQueryBuilder.must(termsAggregationBuilder);
        }
        sourceBuilder.query(boolQueryBuilder);
        if (pageInfo != null) {
            sourceBuilder.from(pageInfo.getStartNumber()).size(pageInfo.getPageSize());
        }
        searchRequest.indices(connectionName);
        searchRequest.source(sourceBuilder);
        searchRequest.searchType(SearchType.DFS_QUERY_THEN_FETCH);

        SearchResponse searchResponse = null;
        try {
            searchResponse = client.search(searchRequest, RequestOptions.DEFAULT);
        } catch (IOException e) {
            log.error("elastic client.search error"+e.getLocalizedMessage());
            e.printStackTrace();
        }
        log.debug("searchResponse="+searchResponse);

        //解析查询结果
        ElasticPage elasticPage = new ElasticPage();
        long totalSize = searchResponse.getHits().getTotalHits().value;
        Iterator<SearchHit> serarchHits = searchResponse.getHits().iterator();
        List<JSONObject> listResult = new ArrayList<>();
        while(serarchHits.hasNext()) {
            SearchHit searchHit = serarchHits.next();
            JSONObject jsonObject = JSONObject.parseObject(searchHit.getSourceAsString());
            listResult.add(jsonObject);
        }
        log.debug("listResult="+listResult);

        elasticPage.setTotalSize(totalSize);
        elasticPage.setResults(listResult);

        return elasticPage;
    }

    @Override
    public <T extends Serializable> ElasticPage<T> findEntity(String connectionName, Map<String, Object> hashMap, TimeInfo timeInfo, PageInfo pageInfo, Map<String, String> sortMap)
    {
        SearchRequest searchRequest = new SearchRequest();
        SearchSourceBuilder sourceBuilder = new SearchSourceBuilder();

        List<TermsQueryBuilder> list = new ArrayList();
        Iterator localIterator1;
        if ((hashMap != null) && (!(hashMap.isEmpty()))) {
            for (localIterator1 = hashMap.entrySet().iterator(); localIterator1.hasNext(); ) {
                Map.Entry entry = (Map.Entry)localIterator1.next();
                String mapKey = (String)entry.getKey();
                if (entry.getValue() != null) {
                    TermsQueryBuilder termsQueryBuilder = null;
                    if (entry.getValue().getClass() == ArrayList.class) {
                        this.log.debug("mapKey=" + mapKey + ",entry.getValue()=" + entry.getValue());
                        termsQueryBuilder = QueryBuilders.termsQuery(mapKey, (ArrayList)entry.getValue());
                    } else {
                        String mapValue = entry.getValue().toString();
                        termsQueryBuilder = QueryBuilders.termsQuery(mapKey, new String[] { mapValue });
                    }
                    list.add(termsQueryBuilder);
                }
            }
        }

//        Map.Entry entry;
        BoolQueryBuilder boolQueryBuilder = QueryBuilders.boolQuery();

        if ((timeInfo != null) && (StringUtils.isNotEmpty(timeInfo.getStartTime())) && (StringUtils.isNotEmpty(timeInfo.getEndTime()))){
            boolQueryBuilder.must(QueryBuilders.rangeQuery(timeInfo.getFieldName()).gte(timeInfo.getStartTime()).lt(timeInfo.getEndTime()));
        } else if ((timeInfo != null) && (StringUtils.isNotEmpty(timeInfo.getStartTime()))){
            boolQueryBuilder.must(QueryBuilders.rangeQuery(timeInfo.getFieldName()).gte(timeInfo.getStartTime()));
        }
        else if ((timeInfo != null) && (StringUtils.isNotEmpty(timeInfo.getStartTime()))) {
            boolQueryBuilder.must(QueryBuilders.rangeQuery(timeInfo.getFieldName()).lt(timeInfo.getEndTime()));
        }

        for (TermsQueryBuilder termsAggregationBuilder : list) {
            boolQueryBuilder.must(termsAggregationBuilder);
        }
        sourceBuilder.query(boolQueryBuilder);
        if (pageInfo != null) {
            sourceBuilder.from(pageInfo.getStartNumber()).size(pageInfo.getPageSize());
        }
        if ((sortMap != null) && (sortMap.containsKey("sortFieldName")) && (sortMap.containsKey("sortType"))) {
            SortBuilder sortBuilder = SortBuilders.fieldSort((String)sortMap.get("sortFieldName"));
            String sortType = (String)sortMap.get("sortType");
            if ("ASC".equals(sortType)) {
                sortBuilder.order(SortOrder.ASC);
            }else {
                sortBuilder.order(SortOrder.DESC);
            }
            sourceBuilder.sort(sortBuilder);
        }

        searchRequest.indices(new String[] { connectionName });
        searchRequest.source(sourceBuilder);
        searchRequest.searchType(SearchType.DFS_QUERY_THEN_FETCH);

        SearchResponse searchResponse = null;
        ElasticPage elasticPage = new ElasticPage();
        try {
            searchResponse = this.client.search(searchRequest, RequestOptions.DEFAULT);

            this.log.info("searchResponse=" + JSONObject.toJSONString(searchResponse));


            long totalSize = searchResponse.getHits().getTotalHits().value;
            Iterator serarchHits = searchResponse.getHits().iterator();
            List listResult = new ArrayList();
            while (serarchHits.hasNext()) {
                SearchHit searchHit = (SearchHit)serarchHits.next();
                JSONObject jsonObject = JSONObject.parseObject(searchHit.getSourceAsString());
                listResult.add(jsonObject);
            }
            this.log.info("listResult=" + listResult);

            elasticPage.setTotalSize(totalSize);
            elasticPage.setResults(listResult);
        } catch (IOException e) {
            this.log.error("elastic client.search error" + e.getLocalizedMessage());
//            e.printStackTrace();

        }
        return elasticPage;
    }



    public List<Map> countAndSearch(String index, Map<String, Object> hashMap,String groupField,TimeInfo timeInfo,String fuzzySearchStr) throws Exception {
        return countAndGroup(index,hashMap,groupField,timeInfo,fuzzySearchStr);
//        return 0L;
    }

    public long countSearch(String index, Map<String, Object> hashMap,TimeInfo timeInfo,String fuzzySearchStr) throws Exception {


        CountRequest countRequest = new CountRequest(index);
        SearchSourceBuilder builder = new SearchSourceBuilder();
        BoolQueryBuilder boolQueryBuilder = QueryBuilders.boolQuery();

        List<TermsQueryBuilder> list = new ArrayList();
        Iterator localIterator1;
        if ((hashMap != null) && (!(hashMap.isEmpty()))) {
            for (localIterator1 = hashMap.entrySet().iterator(); localIterator1.hasNext(); ) {
                Map.Entry entry = (Map.Entry)localIterator1.next();
                String mapKey = (String)entry.getKey();
                if (entry.getValue() != null) {
                    TermsQueryBuilder termsQueryBuilder = null;
                    if (entry.getValue().getClass() == ArrayList.class) {
                        this.log.debug("mapKey=" + mapKey + ",entry.getValue()=" + entry.getValue());
                        termsQueryBuilder = QueryBuilders.termsQuery(mapKey, (ArrayList)entry.getValue());
                    } else {
                        String mapValue = entry.getValue().toString();
                        termsQueryBuilder = QueryBuilders.termsQuery(mapKey, new String[] { mapValue });
                    }
                    list.add(termsQueryBuilder);
                }
            }
        }

        //等于条件
//        boolQueryBuilder.must(QueryBuilders.termsQuery("resourceId", "31"));

        for (TermsQueryBuilder termsAggregationBuilder : list) {
            boolQueryBuilder.must(termsAggregationBuilder);
        }

        if ((timeInfo != null) && (StringUtils.isNotEmpty(timeInfo.getStartTime())) && (StringUtils.isNotEmpty(timeInfo.getEndTime()))){
            boolQueryBuilder.must(QueryBuilders.rangeQuery(timeInfo.getFieldName()).gte(timeInfo.getStartTime()).lt(timeInfo.getEndTime()));
        } else if ((timeInfo != null) && (StringUtils.isNotEmpty(timeInfo.getStartTime()))){
            boolQueryBuilder.must(QueryBuilders.rangeQuery(timeInfo.getFieldName()).gte(timeInfo.getStartTime()));
        }
        else if ((timeInfo != null) && (StringUtils.isNotEmpty(timeInfo.getStartTime()))) {
            boolQueryBuilder.must(QueryBuilders.rangeQuery(timeInfo.getFieldName()).lt(timeInfo.getEndTime()));
        }

        //不等于条件
//        boolQueryBuilder.mustNot(QueryBuilders.termsQuery("字段" +  ".keyword", "字段值"));

        //时间范围查询
//        boolQueryBuilder.must(QueryBuilders.rangeQuery("字段"+ ".keyword").from("开始时间值"));
//        boolQueryBuilder.must(QueryBuilders.rangeQuery("字段"+ ".keyword").to("结束时间值"));


        //全局模糊查询
        if (StringUtils.isNotBlank(fuzzySearchStr)) {
            boolQueryBuilder.must(QueryBuilders.queryStringQuery("*"+fuzzySearchStr+"*"));
        }

        builder.query(boolQueryBuilder);
        String rootGroupKey = "resourceSpecGroup";
        TermsAggregationBuilder projectGroupTerm = AggregationBuilders.terms(rootGroupKey).field("resourceSpecId");

        builder.aggregation(projectGroupTerm);
        countRequest.source(builder);

        CountResponse countResponse = client.count(countRequest, RequestOptions.DEFAULT);
        long count = countResponse.getCount();
        return count;
    }


    /**
     * 分组统计
     * @param index
     * @param hashMap
     * @param groupField
     * @param timeInfo
     * @param fuzzySearchStr
     * @return
     * @throws Exception
     */
    private List<Map> countAndGroup(String index, Map<String, Object> hashMap,String groupField ,TimeInfo timeInfo,String fuzzySearchStr) throws Exception {
        try{

            // 1、创建search请求
            //SearchRequest searchRequest = new SearchRequest();
            SearchRequest searchRequest = new SearchRequest(index);

            // 2、用SearchSourceBuilder来构造查询请求体 ,请仔细查看它的方法，构造各种查询的方法都在这。
            SearchSourceBuilder sourceBuilder = new SearchSourceBuilder();


            BoolQueryBuilder boolQueryBuilder = QueryBuilders.boolQuery();

            List<TermsQueryBuilder> list = new ArrayList();
            Iterator localIterator1;
            if ((hashMap != null) && (!(hashMap.isEmpty()))) {
                for (localIterator1 = hashMap.entrySet().iterator(); localIterator1.hasNext(); ) {
                    Map.Entry entry = (Map.Entry)localIterator1.next();
                    String mapKey = (String)entry.getKey();
                    if (entry.getValue() != null) {
                        TermsQueryBuilder termsQueryBuilder = null;
                        if (entry.getValue().getClass() == ArrayList.class) {
                            this.log.debug("mapKey=" + mapKey + ",entry.getValue()=" + entry.getValue());
                            termsQueryBuilder = QueryBuilders.termsQuery(mapKey, (ArrayList)entry.getValue());
                        } else {
                            String mapValue = entry.getValue().toString();
                            termsQueryBuilder = QueryBuilders.termsQuery(mapKey, new String[] { mapValue });
                        }
                        list.add(termsQueryBuilder);
                    }
                }
            }


            for (TermsQueryBuilder termsAggregationBuilder : list) {
                boolQueryBuilder.must(termsAggregationBuilder);
            }

            if ((timeInfo != null) && (StringUtils.isNotEmpty(timeInfo.getStartTime())) && (StringUtils.isNotEmpty(timeInfo.getEndTime()))){
                boolQueryBuilder.must(QueryBuilders.rangeQuery(timeInfo.getFieldName()).gte(timeInfo.getStartTime()).lt(timeInfo.getEndTime()));
            } else if ((timeInfo != null) && (StringUtils.isNotEmpty(timeInfo.getStartTime()))){
                boolQueryBuilder.must(QueryBuilders.rangeQuery(timeInfo.getFieldName()).gte(timeInfo.getStartTime()));
            }
            else if ((timeInfo != null) && (StringUtils.isNotEmpty(timeInfo.getStartTime()))) {
                boolQueryBuilder.must(QueryBuilders.rangeQuery(timeInfo.getFieldName()).lt(timeInfo.getEndTime()));
            }

            sourceBuilder.query(boolQueryBuilder);
            //构造QueryBuilder
//            QueryBuilder matchQueryBuilder = QueryBuilders.matchQuery("resourceSpecId.keyword", "web_event_srv.upload");
//            sourceBuilder.query(matchQueryBuilder);
//            sourceBuilder.size(0);

            //加入聚合
            //字段值项分组聚合
            TermsAggregationBuilder aggregation = AggregationBuilders.terms(groupField+"group")
                    .field(groupField);//.order(BucketOrder.aggregation("count", true));
            //计算每组的平均balance指标
//            aggregation.subAggregation(AggregationBuilders.count("count")
//                    .field("sessionid"));
            sourceBuilder.aggregation(aggregation);

            searchRequest.source(sourceBuilder);

            //3、发送请求
            SearchResponse searchResponse = client.search(searchRequest,RequestOptions.DEFAULT);

            //4、处理响应
            //搜索结果状态信息
            List<Map> result=new ArrayList<>();
            if(RestStatus.OK.equals(searchResponse.status())) {
                // 获取聚合结果
                Aggregations aggregations = searchResponse.getAggregations();
                Terms byAgeAggregation = aggregations.get(groupField+"group");
                for(Terms.Bucket buck : byAgeAggregation.getBuckets()) {
                    Map map=new HashMap();
                    map.put("name",buck.getKeyAsString());
                    //取子聚合
//                    ParsedValueCount averageBalance = buck.getAggregations().get("count");

                    map.put("count",buck.getDocCount());
                    result.add(map);
                }
            }
            return result;
        }catch (Exception e){
            e.printStackTrace();
            return null;
        }

    }

    /**
     * 分组统计
     * @param index
     * @param hashMap
     * @param groupField
     * @param timeInfo
     * @param fuzzySearchStr
     * @return
     * @throws Exception
     */
    @Override
    public Map countAndGroupWhitEventTime(String index, Map<String, Object> hashMap,String groupField ,TimeInfo timeInfo,String fuzzySearchStr) throws Exception {
        try{

            SearchRequest searchRequest = new SearchRequest(index);

            SearchSourceBuilder sourceBuilder = new SearchSourceBuilder();

            BoolQueryBuilder boolQueryBuilder = QueryBuilders.boolQuery();

            List<TermsQueryBuilder> list = new ArrayList();
            Iterator localIterator1;
            if ((hashMap != null) && (!(hashMap.isEmpty()))) {
                for (localIterator1 = hashMap.entrySet().iterator(); localIterator1.hasNext(); ) {
                    Map.Entry entry = (Map.Entry)localIterator1.next();
                    String mapKey = (String)entry.getKey();
                    if (entry.getValue() != null) {
                        TermsQueryBuilder termsQueryBuilder = null;
                        if (entry.getValue().getClass() == ArrayList.class) {
                            this.log.debug("mapKey=" + mapKey + ",entry.getValue()=" + entry.getValue());
                            termsQueryBuilder = QueryBuilders.termsQuery(mapKey, (ArrayList)entry.getValue());
                        } else {
                            String mapValue = entry.getValue().toString();
                            termsQueryBuilder = QueryBuilders.termsQuery(mapKey, new String[] { mapValue });
                        }
                        list.add(termsQueryBuilder);
                    }
                }
            }


            for (TermsQueryBuilder termsAggregationBuilder : list) {
                boolQueryBuilder.must(termsAggregationBuilder);
            }

            if ((timeInfo != null) && (StringUtils.isNotEmpty(timeInfo.getStartTime())) && (StringUtils.isNotEmpty(timeInfo.getEndTime()))){
                boolQueryBuilder.must(QueryBuilders.rangeQuery(timeInfo.getFieldName()).gte(timeInfo.getStartTime()).lt(timeInfo.getEndTime()));
            } else if ((timeInfo != null) && (StringUtils.isNotEmpty(timeInfo.getStartTime()))){
                boolQueryBuilder.must(QueryBuilders.rangeQuery(timeInfo.getFieldName()).gte(timeInfo.getStartTime()));
            }
            else if ((timeInfo != null) && (StringUtils.isNotEmpty(timeInfo.getStartTime()))) {
                boolQueryBuilder.must(QueryBuilders.rangeQuery(timeInfo.getFieldName()).lt(timeInfo.getEndTime()));
            }

            sourceBuilder.query(boolQueryBuilder);
            //构造QueryBuilder
//            QueryBuilder matchQueryBuilder = QueryBuilders.matchQuery("resourceSpecId.keyword", "web_event_srv.upload");
//            sourceBuilder.query(matchQueryBuilder);
//            sourceBuilder.size(0);

            List<BucketOrder> orders = new ArrayList<>();

            orders.add( BucketOrder.key(false));
            orders.add( BucketOrder.count(false));


            DateHistogramAggregationBuilder timeFieldBuilder = AggregationBuilders.dateHistogram("groupDate").field("eventTime").dateHistogramInterval(DateHistogramInterval.hours(1));
            //加入聚合
            //字段值项分组聚合
            if(!StringUtils.isEmpty(groupField)){
                TermsAggregationBuilder aggregation = AggregationBuilders.terms(groupField).field(groupField);//.order(BucketOrder.aggregation("count", true));
                //根据时间分组统计总数

                aggregation.subAggregation(timeFieldBuilder).size(12).order();
                sourceBuilder.aggregation(aggregation).size(12);
            }else{
                sourceBuilder.aggregation(timeFieldBuilder).size(12);
            }
             //计算每组的平均balance指标


            searchRequest.source(sourceBuilder);

            //3、发送请求
            SearchResponse searchResponse = client.search(searchRequest,RequestOptions.DEFAULT);

            //4、处理响应
            //搜索结果状态信息
            List<Map> result=new ArrayList<>();
            Map countMap = new HashMap();
            Map<String, Map<String, Long>> map = new HashMap<>();
            if(RestStatus.OK.equals(searchResponse.status())) {
                // 获取聚合结果
                Aggregations aggregations = searchResponse.getAggregations();


                if(!StringUtils.isEmpty(groupField)){
                    ParsedStringTerms terms1 = aggregations.get(groupField);
                    for (Terms.Bucket bucket1 : terms1.getBuckets()) {
                        ParsedDateHistogram terms2 = bucket1.getAggregations().get("groupDate");
                        Map<String, Long> map1 = new HashMap<>();
                        for (Histogram.Bucket bucket2 : terms2.getBuckets()) {
                            map1.put(bucket2.getKeyAsString(), bucket2.getDocCount());
                        }
                        map.put(bucket1.getKeyAsString(), map1);
                    }
                }else{
                    ParsedDateHistogram terms2 = aggregations.get("groupDate");
                    Map<String, Long> map1 = new HashMap<>();
                    for (Histogram.Bucket bucket2 : terms2.getBuckets()) {
                        map1.put(bucket2.getKeyAsString(), bucket2.getDocCount());
                        map.put("eventData",map1);
                    }
                }


            }
            return map;
        }catch (Exception e){
            e.printStackTrace();
            return null;
        }


    }



    void search3(String connectionName) throws Exception{
        SearchRequest searchRequest = new SearchRequest(connectionName);
        SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
        // 过滤数据 where，size 0不返回数据，只要统计
        searchSourceBuilder.query(QueryBuilders.matchAllQuery()).size(0);

        // 对上面过滤后的数据，group by 分组统计，如果是字符，用.keyword 数字不加
//        TermsAggregationBuilder groupBy = AggregationBuilders.terms("org1").field("lastFirstOrgNo.keyword")
//                             .order(BucketOrder.compound(BucketOrder.count(false))); // 对group后的排序
//        searchSourceBuilder.aggregation(groupBy);

        // group by 2个字段
        TermsAggregationBuilder groupByOrg1 = AggregationBuilders.terms("org1").field("resourceSpecId.keyword");
//        TermsAggregationBuilder groupByOrg1 = AggregationBuilders.terms("org1").field("lastFirstOrgNo.keyword").subAggregation(groupByOrg2);
        searchSourceBuilder.aggregation(groupByOrg1);

        searchRequest.source(searchSourceBuilder);
        SearchResponse searchResponse = client.search(searchRequest, RequestOptions.DEFAULT);
        SearchHits hits = searchResponse.getHits();
        System.out.println(hits.getTotalHits()); //总数

        Terms terms = searchResponse.getAggregations().get("org1");
        List<? extends Terms.Bucket> buckets = terms.getBuckets();
        for (Terms.Bucket bucket : buckets) {
//            Terms terms2 = bucket.getAggregations().get("org2");

            System.out.println(bucket.getKey()+" "+  bucket.getKey() +" ("+bucket.getDocCount()+")");
//            List<? extends Terms.Bucket> buckets2 = terms2.getBuckets();
//            for (Terms.Bucket bucket2 : buckets2) {
//                System.out.println(bucket.getKey()+" "+  bucket2.getKey() +" ("+bucket2.getDocCount()+")");
//            }
        }
    }



}
